package datapacket.test;

import datapacket.*;
import junit.framework.TestCase;
import java.util.Vector;

/**
 * JUnit tests for the datapacket package
 * @author Stephen Wong (c) 2010
 *
 */
public class DataPacketTest extends TestCase {

	
	public void testDataPacket() {
		ADataPacket ds = new DataPacket<String>(String.class, "Stuff");
		ADataPacket di = new DataPacket<Integer>(Integer.class, 42);
		ADataPacket dd = new DataPacket<Double>(Double.class, 3.1415926);

		// make the vector of data packets
		VDataPacket vdp = new VDataPacket();
		vdp.add(ds);
		vdp.add(di);
		vdp.add(dd);
		
		// make the composite data packet
		ADataPacket vd = new DataPacket<VDataPacket>(VDataPacket.class, vdp);
		
		System.out.println("class = "+ vd.getClass());
		
		final DataPacketAlgo<String, String> algo = new DataPacketAlgo<String, String>(new ADataPacketAlgoCmd<String, Object, String>(){

			private static final long serialVersionUID = -3838770346677745909L;

			@Override
			public String apply(DataPacket<Object> host, String... params) {
				Object x = host.getData();
				return "Default case called. data = "+ x;
			}	
		});
				
		
		algo.setCmd(String.class, new ADataPacketAlgoCmd<String, String, String>(){

			private static final long serialVersionUID = 7417327345957770087L;

			@Override
			public String apply(DataPacket<String> host, String... params) {
				String x = host.getData();
				return "String case called. data = "+ x;
			}
		});
		
		algo.setCmd(Integer.class, new ADataPacketAlgoCmd<String, Integer, String>(){

			private static final long serialVersionUID = 5948981304362218691L;

			@Override
			public String apply(DataPacket<Integer> host, String... params) {
				Integer x = host.getData();
				return "Integer case called. data = "+x;
			}
		});
		
		
		algo.setCmd(VDataPacket.class, new ADataPacketAlgoCmd<String, VDataPacket, String>(){

			private static final long serialVersionUID = -5626422695894697578L;

			private ADataPacketAlgoCmd<Vector<String>, VDataPacket, String> cmd = VDataPacket.makeMapCmd(algo);
			@Override
			public String apply(DataPacket<VDataPacket> host, String... params) {
				Vector<String> results = cmd.apply(host, params);
				String result = "{\n";
				for(String s: results ){
					result += s + "\n";
				}
				result += "}";
				return "Composite case called. result = "+result;
			}
		});
		
		String s = ds.execute(algo);
		assertEquals("ds.execute(algo)","String case called. data = Stuff", s);
		System.out.println("ds.execute(algo) = "+s);
		s = di.execute(algo);
		assertEquals("di.execute(algo)","Integer case called. data = 42", s);
		System.out.println("di.execute(algo) = "+s);
		s = dd.execute(algo);
		assertEquals("dd.execute(algo)","Default case called. data = 3.1415926", s);
		System.out.println("dd.execute(algo) = "+s);
		s = vd.execute(algo);
		assertEquals("vd.execute(algo)", "Composite case called. result = {\nString case called. data = Stuff\nInteger case called. data = 42\nDefault case called. data = 3.1415926\n}", s);
		System.out.println("vd.execute(algo) = "+s);
		
		
	}

}
