package status;


import extvisitor.*;
/**
 * Convenience class to simplify the code and to tighten the type checking on status processing code.
 * @author Stephen Wong (c) 2010
 *
 * @param <R>   The return type of the algorithm.
 * @param <S>  The concrete type of the status host.
 * @param <P>  The type of the input vararg parameters
 */
public abstract class AStatusAlgoCmd<R, S extends AStatus, P> implements IExtVisitorCmd<R, Class<? extends AStatus>, P, AStatus>{
	
	private static final long serialVersionUID = 2465721999672809380L;

	/**
	 * The actual method called by the host visitor when the associated case is invoked.   
	 * This method simply forwards the call to the abstract apply method, performing 
	 * an unchecked cast of the host to the required DataPacket type.
	 * @param index  The String object used to identify the host
	 * @param host The host calling the visitor
	 * @param params Vararg input parameters to be used for processing the host
	 * @return The result of this case.
	 */
	@SuppressWarnings("unchecked")
	final public <T extends IExtVisitorHost<Class<? extends AStatus>, ? super AStatus>> R apply(Class<? extends AStatus> index, T host, P... params) {
		return apply((S) host, params);
	}

	/**
	 * Abstract method that actually performs the processing of the case.   
	 * Here, the host is strongly typed to be the AStatus sub-type appropriate for the case.
	 * @param host  The AStatus subclass host calling the visitor
	 * @param params  Vararg input parameter to be used for processing the host
	 * @return  The result of this case.
	 */
	abstract public R apply(S host, P...params);

}
