package status;

/**
 * Concrete status object that represents a failure.
 * @author Stephen Wong (c) 2010
 *
 */
public class FailStatus extends AStatus {
	
	private static final long serialVersionUID = 972560608092286364L;

	/**
	 * Enumeration that distinguishes between different failure levels.
	 */
	public static enum LEVEL {MILD, MEDIUM, SEVERE};
	
	/**
	 * The failure message
	 */
	private String msg = "Unknown failure.";

	/**
	 * The machine where the failure occurred.
	 */
	private String src = "Unknown source machine";
	
	/**
	 * The severity level of this failure.
	 */
	private LEVEL failLevel = LEVEL.MEDIUM;
	
	/**
	 * Constructor for the class.
	 * @param src  The machine where the failure occurred.
	 * @param msg  The failure message
	 * @param failLevel  The severity level of this failure.
	 */
	public FailStatus(String src, String msg, LEVEL failLevel) {
		super(FailStatus.class);
		this.src = src;
		this.msg = msg;
		this.failLevel = failLevel;
	}
	
	/**
	 * Get the failure messsage
	 * @return
	 */
	public String getMsg() {
		return msg;
	}
	
	/**
	 * Get the failure's source machine
	 * @return
	 */
	public String getSrc() {
		return src;
	}
	
	/**
	 * Get the severity level of the failure.
	 * @return
	 */
	public LEVEL getLevel(){
		return failLevel;
	}
	
	public String toString(){
		return "Status: ["+failLevel+"] "+src+" --> "+msg;
	}

}
