package status.test;

import status.*;
import junit.framework.TestCase;

public class StatusTest extends TestCase {

	public void testStatus() {
		AStatus ok = OkStatus.Singleton;
		AStatus fail = new FailStatus("Test machine", "Strange failure.", FailStatus.LEVEL.SEVERE);
		AStatus info = new InfoStatus("This is a just a bit of information.");
		
		
		StatusAlgo<String, String> algo = new StatusAlgo<String, String>(new AStatusAlgoCmd<String, AStatus, String>(){

			private static final long serialVersionUID = -3838770346677745909L;

			@Override
			public String apply(AStatus host, String... params) {
				return "Default case called. Result = "+ host;
			}
	
		});
				
		
		algo.setCmd(FailStatus.class, new AStatusAlgoCmd<String, FailStatus, String>(){

			private static final long serialVersionUID = 7417327345957770087L;

			@Override
			public String apply(FailStatus host, String... params) {
				return "Fail case called. data = ["+ host.getLevel()+"] on "+ host.getSrc()+" due to "+host.getMsg();
			}
		});
		
		algo.setCmd(InfoStatus.class, new AStatusAlgoCmd<String, InfoStatus, String>(){

			private static final long serialVersionUID = 5948981304362218691L;

			@Override
			public String apply(InfoStatus host, String... params) {
				return "Msg = "+host.getMsg();
			}
		});
		
		String s = ok.execute(algo);
		assertEquals("ok.execute(algo)","Default case called. Result = Status: OK", s);
		System.out.println("ok.execute(algo) = "+s);
		s = fail.execute(algo);
		assertEquals("fail.execute(algo)","Fail case called. data = [SEVERE] on Test machine due to Strange failure.", s);
		System.out.println("fail.execute(algo) = "+s);
		s = info.execute(algo);
		assertEquals("info.execute(algo)","Msg = This is a just a bit of information.", s);
		System.out.println("info.execute(algo) = "+s);
		
	}
	
}
